// __________________________________________________________________________________________________
//    GLGooey Graphical User Interface for OpenGL
//    Copyright (c) 2004 Niel Waldren
//
// This software is provided 'as-is', without any express or implied warranty. In no event will
// the authors be held liable for any damages arising from the use of this software.
//
// Permission is granted to anyone to use this software for any purpose, including commercial
// applications, and to alter it and redistribute it freely, subject to the following restrictions:
//
//     1. The origin of this software must not be misrepresented; you must not claim that you
//        wrote the original software. If you use this software in a product, an acknowledgment
//        in the product documentation would be appreciated but is not required.
//
//     2. Altered source versions must be plainly marked as such, and must not be misrepresented
//        as being the original software.
//
//     3. This notice may not be removed or altered from any source distribution.
//
// __________________________________________________________________________________________________
//
//  Author(s):
//    Sebastien Berthet (sbrt@yahoo.fr)
//
// __________________________________________________________________________________________________
#ifndef DOCK__H
#define DOCK__H

#include "glgooey/core/Core.h"

#include "glgooey/core/Vector2.h"
#include "glgooey/core/Alignment.h"



// *************************************************************************************************
namespace Gooey
{


// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
class Dock
{
public:    
    Dock() : docked_(false), horizontalStretch_(false), verticalStretch_(false) {}
    virtual ~Dock() {}

    void setDocked(bool d) { docked_=d; }
    void setHorizontalAlignment(Core::Alignment::Horizontal h)  { horizontal_=h; }
    void setStretch(bool horizontal, bool vertical) { horizontalStretch_ = horizontal; verticalStretch_ = vertical; }
    void setVerticalAlignment(Core::Alignment::Vertical v) { vertical_=v; }
    void setMargin(const Core::Vector2& m) { margin_=m; }

    bool docked() const { return docked_; }
    Core::Alignment::Vertical verticalAlignment()  const { return vertical_;  }
    Core::Alignment::Horizontal horizontalAlignment() const { return horizontal_; }    
    bool horizontalStretch() const { return horizontalStretch_; }
    bool verticalStretch() const { return verticalStretch_; }
    const Core::Vector2& margin() const { return margin_; }

    void dock(Core::Alignment::Horizontal h, Core::Alignment::Vertical v, const Core::Vector2& m)
    { setDocked(true); setVerticalAlignment(v); setHorizontalAlignment(h); setMargin(m);    }
    
    void undock()
    { setDocked(false); }

    void computePosition(Core::Vector2& position, const Core::Vector2& size, const Core::Vector2& parentSize) const;
    bool computeSize(Core::Vector2& newSize, const Core::Vector2& oldSize, const Core::Vector2& parentSize) const;

private:
    bool docked_;
    Core::Alignment::Vertical vertical_;
    Core::Alignment::Horizontal horizontal_;
    bool horizontalStretch_;
    bool verticalStretch_;
    Core::Vector2 margin_;
};
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<


// *************************************************************************************************
} // namespace

#endif
