// __________________________________________________________________________________________________
//    GLGooey Graphical User Interface for OpenGL
//    Copyright (c) 2004 Niel Waldren
//
// This software is provided 'as-is', without any express or implied warranty. In no event will
// the authors be held liable for any damages arising from the use of this software.
//
// Permission is granted to anyone to use this software for any purpose, including commercial
// applications, and to alter it and redistribute it freely, subject to the following restrictions:
//
//     1. The origin of this software must not be misrepresented; you must not claim that you
//        wrote the original software. If you use this software in a product, an acknowledgment
//        in the product documentation would be appreciated but is not required.
//
//     2. Altered source versions must be plainly marked as such, and must not be misrepresented
//        as being the original software.
//
//     3. This notice may not be removed or altered from any source distribution.
//
// __________________________________________________________________________________________________
#ifndef __GOOEY_EDIT_PANEL_H__
#define __GOOEY_EDIT_PANEL_H__


#include "glgooey/EditWindow.h"

#include "glgooey/RenderDesc.h"



// *************************************************************************************************
//  Forward declarations
namespace Gooey
{
    class ScrollBar;
}
// *************************************************************************************************




// *************************************************************************************************
namespace Gooey
{



// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
//! \brief Edit panels allow the user to edit multiple lines of  text
class EditPanel : public EditWindow
{
    // ---------------------------------------------------------------------------------------------
    //  Construction / Destruction
    // ---------------------------------------------------------------------------------------------
public:
    //! \brief constructor requires the parent and the text with which the panel
    //! is to be initialized
    EditPanel(Window* aParent, const std::string& aText);
    
    //! Destructor
    virtual ~EditPanel();
    
    
    // ---------------------------------------------------------------------------------------------
    //  Public Interface
    // ---------------------------------------------------------------------------------------------
public:
    //! Reacts to the return key being pressed by inserting a '\n' character
    EditPanel& onReturnPressed();

    //! Returns the height of a line in pixels
    float lineHeight() const;

    //! Puts the scroll bar in the right place and shows or hides it as required
    EditPanel& positionScrollBar();

    //! Sets the scroll bar's parameters according to the contents of the edit panel
    void setScrollBarParameters();

    //! Returns the total height of all lines in pixels
    float heightOfAllContents() const;
    
    //! The smaller of the window's height and the height of all contents
    float heightOfVisibleContents() const;
    
    //! React to the text changing
    virtual void onTextChanged();
    
    //! Called when the window is resized  (resizes the scroll bar)
    virtual void onResize();

    //! Called when the window is moved (repositions the scroll bar)
    virtual void onMove();

    //! Returns true if this window's sub-hierarchy contains the passed in window   
    virtual bool doesHierarchyContain(Window* aWindow) const;

    //! Returns the window which is below the passed in point
    virtual Window* windowBelow(const Core::Vector2& aPoint);

    
    // ---------------------------------------------------------------------------------------------
    //  Implementation
    // ---------------------------------------------------------------------------------------------
protected:    
    //! Draws the edit panel
    virtual Window& draw();
    
    //! This is overridden in order to take the scroll bar into account
    virtual Core::Rectangle drawRectangle() const;
    
    //! Returns the index of the first visible line
    virtual size_t firstVisibleLine() const;

    //! \brief Reacts to the caret moving by changing the visible text if necessary, ensuring that
    //! the caret remains visible
    virtual EditWindow& onCaretMoved(size_t pos);
    
    
private:
    //! Returns the number of lines that fit in the window
    size_t numberOfVisibleLines() const;

    ScrollBar* _verticalScrollBar; //!< the vertical scroll bar
};
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<


// *************************************************************************************************
} // namespace




#endif
