// __________________________________________________________________________________________________
//    GLGooey Graphical User Interface for OpenGL
//    Copyright (c) 2004 Niel Waldren
//
// This software is provided 'as-is', without any express or implied warranty. In no event will
// the authors be held liable for any damages arising from the use of this software.
//
// Permission is granted to anyone to use this software for any purpose, including commercial
// applications, and to alter it and redistribute it freely, subject to the following restrictions:
//
//     1. The origin of this software must not be misrepresented; you must not claim that you
//        wrote the original software. If you use this software in a product, an acknowledgment
//        in the product documentation would be appreciated but is not required.
//
//     2. Altered source versions must be plainly marked as such, and must not be misrepresented
//        as being the original software.
//
//     3. This notice may not be removed or altered from any source distribution.
//
// __________________________________________________________________________________________________
#ifndef PROGRESSBAR__H
#define PROGRESSBAR__H

#include "glgooey/core/Enumeration.h"

#include "glgooey/Window.h"
#include "glgooey/RenderDesc.h"


// *************************************************************************************************
//  Forward declarations
namespace Gooey
{
    namespace Core
    {
        class Rectangle;
    }
    DEFINE_SMARTPOINTER(ProgressBarDesc);
}
// *************************************************************************************************





// *************************************************************************************************
namespace Gooey
{


// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
//! \brief A simple progress bar.
class ProgressBar : public Window
{
    // ---------------------------------------------------------------------------------------------
    //  Construction / Destruction
    // ---------------------------------------------------------------------------------------------
public:
    //! \brief Constructs a progress bar
    //!
    //! Constructor requires the parent window and the rectangle which is to
    //! be filled with the progress bar. The orientation may also be
    //! specified. If no orientation is specified, the bar will be horizontal
    //! an indicate progress from left to right
    ProgressBar(Window* aParent, const Core::Rectangle& aRectangle, Core::Orientation anOrientation = Core::horizontal);

    //! destructor
    virtual ~ProgressBar();


    // ---------------------------------------------------------------------------------------------
    //  Accessors
    // ---------------------------------------------------------------------------------------------
public:
    //! returns the orientation of the bar
    Core::Orientation orientation() const { return orientation_; }

    //! sets the orientation of the bar
    ProgressBar& setOrientation(Core::Orientation o)
    {
        orientation_ = o;
        return *this;
    }

    //! returns the percentage value that the progress bar is to represent
    float percentage() const { return percentage_; }

    //! sets the percentage value that the progress bar is to represent
    ProgressBar& setPercentage(float p)
    {
        percentage_ = p;
        return *this;
    }



    // ---------------------------------------------------------------------------------------------
    //  Public Interface
    // ---------------------------------------------------------------------------------------------
public:
    // Commented in base class
    virtual Core::Vector2 recommendedSize() const;

    // Commented in base class
    virtual Window& loadAppearance(Core::Archive& anArchive, const std::string& aWindowName);



    // ---------------------------------------------------------------------------------------------
    //  Implementation
    // ---------------------------------------------------------------------------------------------
protected:
    //! render the window to the current OpenGL context
    virtual Window& draw();

private:
    Core::Orientation orientation_;
    float percentage_;

    ProgressBarDescPtr renderDesc_;
    
    
    
    // ---------------------------------------------------------------------------------------------
    //  Serialization
    // ----------------------------------------------------------------------------------------------
    DECLARE_SERIALIZATION;
};
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<








// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
//! \brief The render desc class for progress bars
class ProgressBarDesc : public RenderDesc
{
    // ---------------------------------------------------------------------------------------------
    //  Construction / Destruction
    // ---------------------------------------------------------------------------------------------
public:
    //! Default Constructor
    ProgressBarDesc();

    //! Destructor
    virtual ~ProgressBarDesc();


    // ---------------------------------------------------------------------------------------------
    //  Public Interface
    // ---------------------------------------------------------------------------------------------
public:
    //! draws on the passed in area according to the desc
    virtual void draw(const Core::Rectangle& area) const;

    //! sets the button object that will be drawn on the next call to draw
    ProgressBarDesc& setProgressBar(ProgressBar* aProgressBar)
    {
        progressBar_ = aProgressBar;
        return *this;
    }


private:
    RenderDescPtr  horzEmptyDesc_;
    RenderDescPtr  horzFullDesc_;
    RenderDescPtr  vertEmptyDesc_;
    RenderDescPtr  vertFullDesc_;
    RenderDescPtr  border_;
    ProgressBar* progressBar_;



    // ---------------------------------------------------------------------------------------------
    //  Serialization
    // ----------------------------------------------------------------------------------------------
    DECLARE_SERIALIZATION;
};
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<


// *************************************************************************************************
} // namespace Gooey



#endif
