// __________________________________________________________________________________________________
//    GLGooey Graphical User Interface for OpenGL
//    Copyright (c) 2004 Niel Waldren
//
// This software is provided 'as-is', without any express or implied warranty. In no event will
// the authors be held liable for any damages arising from the use of this software.
//
// Permission is granted to anyone to use this software for any purpose, including commercial
// applications, and to alter it and redistribute it freely, subject to the following restrictions:
//
//     1. The origin of this software must not be misrepresented; you must not claim that you
//        wrote the original software. If you use this software in a product, an acknowledgment
//        in the product documentation would be appreciated but is not required.
//
//     2. Altered source versions must be plainly marked as such, and must not be misrepresented
//        as being the original software.
//
//     3. This notice may not be removed or altered from any source distribution.
//
// __________________________________________________________________________________________________

#include <fstream>
#include <iterator>

#include "glgooey/PropertyScheme.h"

#include "glgooey/core/StandardException.h"
#include "glgooey/core/Serialization.h"


// *************************************************************************************************
namespace Gooey
{



// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
PropertyScheme::PropertyScheme()
{
}
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<






// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
PropertyScheme::~PropertyScheme()
{
}
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<






// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
PropertyScheme&
PropertyScheme::load(const std::string& aFileName)
{
    Core::Archive *ar = Core::ArchiveFactoryInterface::createArchive(aFileName, Core::ArchiveFactoryInterface::read);
    if(!ar) return *this;

    if(!ar->open(aFileName, true)) return *this;
    
    *ar & Core::ObjectNamer<PropertyScheme>("Scheme", *this);
    
    ar->close();
    delete ar;

    return *this;
}
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<






// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
PropertyScheme&
PropertyScheme::save(const std::string& aFileName)
{        
    Core::Archive *ar = Core::ArchiveFactoryInterface::createArchive(aFileName, Core::ArchiveFactoryInterface::write);
    if(!ar) return *this;

    if(!ar->open(aFileName, true)) return *this;
    
    *ar & Core::ObjectNamer<PropertyScheme>("Scheme", *this);
    
    ar->close();
    delete ar;

    return *this;    
}
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<






// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
void
PropertyScheme::serialize(Core::Archive& anArchive)
{
    RenderDesc::serialize(anArchive);    
    anArchive &
        TAGGED_OBJECT(buttonDesc) &
        TAGGED_OBJECT(panelDesc) &
        TAGGED_OBJECT(editWindowDesc) &
        TAGGED_OBJECT(scrollBarDesc) &
        TAGGED_OBJECT(listBoxDesc) &
        TAGGED_OBJECT(listControlDesc) &
        TAGGED_OBJECT(progressBarDesc) &
        TAGGED_OBJECT(checkBoxDesc) &
        TAGGED_OBJECT(sliderDesc) &
        TAGGED_OBJECT(frameWindowDesc) &
        TAGGED_OBJECT(staticTextDesc) &
        TAGGED_OBJECT(comboBoxDesc);
}
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<







// *************************************************************************************************
} //namespace Gooey
