//  Author(s):
//    Jan Becker <tahoma@gmx.de>
// __________________________________________________________________________________________________
//    GLGooey Graphical User Interface for OpenGL
//    Copyright (c) 2004 Niel Waldren
//
// This software is provided 'as-is', without any express or implied warranty. In no event will
// the authors be held liable for any damages arising from the use of this software.
//
// Permission is granted to anyone to use this software for any purpose, including commercial
// applications, and to alter it and redistribute it freely, subject to the following restrictions:
//
//     1. The origin of this software must not be misrepresented; you must not claim that you
//        wrote the original software. If you use this software in a product, an acknowledgment
//        in the product documentation would be appreciated but is not required.
//
//     2. Altered source versions must be plainly marked as such, and must not be misrepresented
//        as being the original software.
//
//     3. This notice may not be removed or altered from any source distribution.
//
// __________________________________________________________________________________________________
#ifndef __GOOEY_BITMAP__H_
#define __GOOEY_BITMAP__H_

#include "mmgr/nommgr.h"
#include <cstddef>
#include <iosfwd>
#include "glgooey/core/Core.h"

#include "glgooey/core/Position.h"
#include "glgooey/core/Vector2.h"


// *************************************************************************************************
namespace Gooey
{


// *************************************************************************************************
namespace Core
{


// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
//! \brief bitmap class.
class Bitmap
{
    // ---------------------------------------------------------------------------------------------
    //  Xonstruction
    // ---------------------------------------------------------------------------------------------
public:
    Bitmap(const Size&);
    
    //! Copy constructor
    Bitmap(const Bitmap& aBitmap) { (*this) = aBitmap; }

    ~Bitmap();
    
    
// ---------------------------------------------------------------------------------------------
    //  Public Interface
    // ---------------------------------------------------------------------------------------------
public:
    //! Returns whether the bit on position x/y is set or not.
    //! If x is greater than width or y is greater than height, 
    //! false is returned.
    bool isSet(const Position&) const;
    
    //! Returns whether the bit on position x/y is set or not.
    //! x and y are normalized values where 1 is the maximum.
    //! If x or y is not in [0,1], false is returned.
    bool isSet(const Vector2&) const;
    
       
// ---------------------------------------------------------------------------------------------
    //  Operators
    // ---------------------------------------------------------------------------------------------
public:
    Bitmap& operator = (const Bitmap& aBitmap); //!< Assignment


// ---------------------------------------------------------------------------------------------
    //  Accessors
    // ---------------------------------------------------------------------------------------------
public:
    //! Returns the bitmap's size
    inline Size size() const { return size_; }
    
    //! Resizes the bitmap. If there was an
    //! error, false is returned.
    bool resize(const Size&);
    
    //! Empty the bitmap.
    void clear();

    //! Sets the value at index x + y*width
    //! Returns the state of the old value.
    bool set(size_t index, int value);

    //! Sets the value at position x/y
    //! Returns the state of the old value.
    bool set(const Position&, int value);
    
    //! Sets the value at normalized position x/y
    //! Returns the state of the old value.
    bool set(const Vector2&, int value);
    


// ---------------------------------------------------------------------------------------------
    //  Implementation
    // ----------------------------------------------------------------------------------------------
private:
    unsigned char* data_; //!< Bitmap data
    Size size_; //!< Size of bitmap


    // ---------------------------------------------------------------------------------------------
    //  Serialization
    // ----------------------------------------------------------------------------------------------
public:
    void serialize(Gooey::Core::Archive& anArchive);
    
};


// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<



// *************************************************************************************************
} // namespace Core

// *************************************************************************************************
} // namespace Gooey


#endif
