// __________________________________________________________________________________________________
//    GLGooey Graphical User Interface for OpenGL
//    Copyright (c) 2004 Niel Waldren
//
// This software is provided 'as-is', without any express or implied warranty. In no event will
// the authors be held liable for any damages arising from the use of this software.
//
// Permission is granted to anyone to use this software for any purpose, including commercial
// applications, and to alter it and redistribute it freely, subject to the following restrictions:
//
//     1. The origin of this software must not be misrepresented; you must not claim that you
//        wrote the original software. If you use this software in a product, an acknowledgment
//        in the product documentation would be appreciated but is not required.
//
//     2. Altered source versions must be plainly marked as such, and must not be misrepresented
//        as being the original software.
//
//     3. This notice may not be removed or altered from any source distribution.
//
// __________________________________________________________________________________________________
#ifndef __GOOEY_KEY_CODES_H__
#define __GOOEY_KEY_CODES_H__

#include <map>

// *************************************************************************************************
namespace Gooey
{


// *************************************************************************************************
namespace Key
{


// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
//! \brief These are the keys that GLGooey understands.
//!
//! It's no coincidence that these map precisely to the SDL keys. This has not only the advantage of
//! making the SDL mapping trivial, but SDL maps as much as it can to ASCII which also simplifies
//! the mappings from other frameworks
typedef enum 
{
    unknown      = 0,
    backspace    = 8,
    tab          = 9,
    clear        = 12,
    ret          = 13,   // can't call this one "return" for obvious reasons
    pause        = 19,
    escape       = 27,
    space        = 32,
    exclamation  = 33,
    doubleQuote  = 34,
    hash         = 35,
    dollar       = 36,
    ampersand    = 38,
    quote        = 39,
    leftBracket  = 40,
    rightBracket = 41,
    asterisk     = 42,
    plus         = 43,
    comma        = 44,
    minus        = 45,
    period       = 46,
    slash        = 47,
    zero         = 48,
    one          = 49,
    two          = 50,
    three        = 51,
    four         = 52,
    five         = 53,
    six          = 54,
    seven        = 55,
    eight        = 56,
    nine         = 57,
    colon        = 58,
    semicolon    = 59,
    less         = 60,
    equals       = 61,
    greater      = 62,
    question     = 63,
    at           = 64,
    // This is where ASCII has the uppercase letters
    leftSquareBracket  = 91,
    backslash          = 92,
    rightSquareBracket = 93,
    caret              = 94,
    underscore         = 95,
    backquote          = 96,
    a                  = 97,
    b                  = 98,
    c                  = 99,
    d                  = 100,
    e                  = 101,
    f                  = 102,
    g                  = 103,
    h                  = 104,
    i                  = 105,
    j                  = 106,
    k                  = 107,
    l                  = 108,
    m                  = 109,
    n                  = 110,
    o                  = 111,
    p                  = 112,
    q                  = 113,
    r                  = 114,
    s                  = 115,
    t                  = 116,
    u                  = 117,
    v                  = 118,
    w                  = 119,
    x                  = 120,
    y                  = 121,
    z                  = 122,
    del                = 127,
    

    world0         = 160,        /* 0xA0 */
    world1         = 161,
    world2         = 162,
    world3         = 163,
    world4         = 164,
    world5         = 165,
    world6         = 166,
    world7         = 167,
    world8         = 168,
    world9         = 169,
    world10        = 170,
    world11        = 171,
    world12        = 172,
    world13        = 173,
    world14        = 174,
    world15        = 175,
    world16        = 176,
    world17        = 177,
    world18        = 178,
    world19        = 179,
    world20        = 180,
    world21        = 181,
    world22        = 182,
    world23        = 183,
    world24        = 184,
    world25        = 185,
    world26        = 186,
    world27        = 187,
    world28        = 188,
    world29        = 189,
    world30        = 190,
    world31        = 191,
    world32        = 192,
    world33        = 193,
    world34        = 194,
    world35        = 195,
    world36        = 196,
    world37        = 197,
    world38        = 198,
    world39        = 199,
    world40        = 200,
    world41        = 201,
    world42        = 202,
    world43        = 203,
    world44        = 204,
    world45        = 205,
    world46        = 206,
    world47        = 207,
    world48        = 208,
    world49        = 209,
    world50        = 210,
    world51        = 211,
    world52        = 212,
    world53        = 213,
    world54        = 214,
    world55        = 215,
    world56        = 216,
    world57        = 217,
    world58        = 218,
    world59        = 219,
    world60        = 220,
    world61        = 221,
    world62        = 222,
    world63        = 223,
    world64        = 224,
    world65        = 225,
    world66        = 226,
    world67        = 227,
    world68        = 228,
    world69        = 229,
    world70        = 230,
    world71        = 231,
    world72        = 232,
    world73        = 233,
    world74        = 234,
    world75        = 235,
    world76        = 236,
    world77        = 237,
    world78        = 238,
    world79        = 239,
    world80        = 240,
    world81        = 241,
    world82        = 242,
    world83        = 243,
    world84        = 244,
    world85        = 245,
    world86        = 246,
    world87        = 247,
    world88        = 248,
    world89        = 249,
    world90        = 250,
    world91        = 251,
    world92        = 252,
    world93        = 253,
    world94        = 254,
    world95        = 255,

    keyPad0        = 256,
    keyPad1        = 257,
    keyPad2        = 258,
    keyPad3        = 259,
    keyPad4        = 260,
    keyPad5        = 261,
    keyPad6        = 262,
    keyPad7        = 263,
    keyPad8        = 264,
    keyPad9        = 265,
    keyPadPeriod   = 266,
    keyPadDivide   = 267,
    keyPadMultiply = 268,
    keyPadMinus    = 269,
    keyPadPlus     = 270,
    keyPadEnter    = 271,
    keyPadEquals   = 272,

    up       = 273,
    down     = 274,
    right    = 275,
    left     = 276,
    insert   = 277,
    home     = 278,
    end      = 279,
    pageUp   = 280,
    pageDown = 281,

    F1  = 282,
    F2  = 283,
    F3  = 284,
    F4  = 285,
    F5  = 286,
    F6  = 287,
    F7  = 288,
    F8  = 289,
    F9  = 290,
    F10 = 291,
    F11 = 292,
    F12 = 293,
    F13 = 294,
    F14 = 295,
    F15 = 296,

    numLock    = 300,
    capsLock   = 301,
    scrollLock = 302,
    rightShift = 303,
    leftShift  = 304,
    rightCtrl  = 305,
    leftCtrl   = 306,
    rightAlt   = 307,
    leftAlt    = 308,
    rightMeta  = 309,
    leftMeta   = 310,
    leftSuper  = 311,       // left windows key
    rightSuper = 312,       // right windows key
    Mode       = 313,       // Alt Gr key
    Compose    = 314,       // Multi-key compose key

    help       = 315,
    print      = 316,
    sysreq     = 317,
    pauseBreak = 318,
    menu       = 319,
    power      = 320,        // Power Macintosh power key
    euro       = 321,        // Some european keyboards
    undo       = 322         // Atari keyboard has Undo

} Code;
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<

// *************************************************************************************************
} // namespace Key





// *************************************************************************************************
namespace Modifier
{


// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
//! \brief These are the key modifiers that GLGooey understands.
//!
//! These also map exactly to their SDL equivalents
typedef enum 
{
    none       = 0x0000,
    leftShift  = 0x0001,
    rightShift = 0x0002,
    shift      = leftShift | rightShift,
    leftCtrl   = 0x0040,
    rightCtrl  = 0x0080,
    ctrl       = leftCtrl | rightCtrl,
    leftAlt    = 0x0100,
    rightAlt   = 0x0200,
    alt        = leftAlt | rightAlt,
    leftMeta   = 0x0400,
    rightMeta  = 0x0800,
    meta       = leftMeta | rightMeta,
    num        = 0x1000,
    caps       = 0x2000,
    mode       = 0x4000,
    reserved   = 0x8000
} Keys;
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<


// *************************************************************************************************
} // namespace Modifier


// *************************************************************************************************
} // namespace Gooey



#endif
