// __________________________________________________________________________________________________
//    GLGooey Graphical User Interface for OpenGL
//    Copyright (c) 2004 Niel Waldren
//
// This software is provided 'as-is', without any express or implied warranty. In no event will
// the authors be held liable for any damages arising from the use of this software.
//
// Permission is granted to anyone to use this software for any purpose, including commercial
// applications, and to alter it and redistribute it freely, subject to the following restrictions:
//
//     1. The origin of this software must not be misrepresented; you must not claim that you
//        wrote the original software. If you use this software in a product, an acknowledgment
//        in the product documentation would be appreciated but is not required.
//
//     2. Altered source versions must be plainly marked as such, and must not be misrepresented
//        as being the original software.
//
//     3. This notice may not be removed or altered from any source distribution.
//
// __________________________________________________________________________________________________
#ifndef __GOOEY_KEY_MAP_H__
#define __GOOEY_KEY_MAP_H__

#include <map>

#include "glgooey/core/KeyCodes.h"

// *************************************************************************************************
namespace Gooey
{




// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
//! \brief Base class for maps which map keys from a particular framework to GLGooey
template<typename KeyType>
class KeyMap
{
    // ---------------------------------------------------------------------------------------------
    //  Data Types
    // ---------------------------------------------------------------------------------------------
private:
    //! Convenience typedef for a map from framework keys to GLGooey keys
    typedef std::map<KeyType, Key::Code> Map;
    
    
    // ---------------------------------------------------------------------------------------------
    //  Xstruction
    // ---------------------------------------------------------------------------------------------
public:
    //! destructor
    virtual ~KeyMap() {}

       
    // ---------------------------------------------------------------------------------------------
    //  Public Interface
    // ---------------------------------------------------------------------------------------------
public:
    //! \brief Returns the GLGooey key associated with the passed in framework key
    //! \note if no mapping has been defined for the passed in key, this method will return 
    //! Key::Code::unknown
    Key::Code toGooeyKey(KeyType aFrameworkKey) const
    {
        typename Map::const_iterator it;
        it = map_.find(aFrameworkKey);
        return (it != map_.end()) ? it->second : Key::unknown;
    }

    //! Call this to add a mapping from a framework key to a GLGooey key to the map
    void addMapping(KeyType aFrameworkKey, Key::Code aGooeyKey) { map_[aFrameworkKey] = aGooeyKey; }

    
    // ---------------------------------------------------------------------------------------------
    //  Implementation
    // ---------------------------------------------------------------------------------------------
private:
    Map map_; //!< The actual map is stored here
};
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<




// *************************************************************************************************
} // namespace Gooey



#endif
