// __________________________________________________________________________________________________
//    GLGooey Graphical User Interface for OpenGL
//    Copyright (c) 2004 Niel Waldren
//
// This software is provided 'as-is', without any express or implied warranty. In no event will
// the authors be held liable for any damages arising from the use of this software.
//
// Permission is granted to anyone to use this software for any purpose, including commercial
// applications, and to alter it and redistribute it freely, subject to the following restrictions:
//
//     1. The origin of this software must not be misrepresented; you must not claim that you
//        wrote the original software. If you use this software in a product, an acknowledgment
//        in the product documentation would be appreciated but is not required.
//
//     2. Altered source versions must be plainly marked as such, and must not be misrepresented
//        as being the original software.
//
//     3. This notice may not be removed or altered from any source distribution.
//
// __________________________________________________________________________________________________
#ifndef __GOOEY_OBJECT__H_
#define __GOOEY_OBJECT__H_

#include "mmgr/nommgr.h"
#include "glgooey/core/SmartPointer.h"
#include "glgooey/core/Serialization.h"
#include "glgooey/core/Core.h"

// *************************************************************************************************
namespace Gooey
{


// *************************************************************************************************
namespace Core
{


DEFINE_SMARTPOINTER(Object);



// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
//! \brief The baseclass for all Gooey classes
class Object
{
    // ---------------------------------------------------------------------------------------------
    //  Class features
    // ---------------------------------------------------------------------------------------------   
    DECLARE_REFEERENCE_COUNTING;
    DECLARE_SERIALIZATION_ROOTCLASS(Object);    


    // ---------------------------------------------------------------------------------------------
    //  Construction
    // ---------------------------------------------------------------------------------------------   
protected:
    Object();
    Object(const std::string& aName);


    // ---------------------------------------------------------------------------------------------
    //  Destruction
    // ---------------------------------------------------------------------------------------------   
public:
    virtual ~Object();
 

    // ---------------------------------------------------------------------------------------------
    //  Public Interface
    // ---------------------------------------------------------------------------------------------
public:
    void setName(const std::string& aName);
    const std::string& name() const;


    // ---------------------------------------------------------------------------------------------
    //  Implementation
    // ----------------------------------------------------------------------------------------------
private:
    std::string name_;
 

    // Because of our protected constructor we make Factory<Object> our friend to make the
    // name based class construction possible.
    friend class Archive;
    friend class FactoryInterface<Object>;   
};


#include "Object.inl"

// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<



// *************************************************************************************************
} // namespace Core

// *************************************************************************************************
} // namespace Gooey


#endif
