// __________________________________________________________________________________________________
//    GLGooey Graphical User Interface for OpenGL
//    Copyright (c) 2004 Niel Waldren
//
// This software is provided 'as-is', without any express or implied warranty. In no event will
// the authors be held liable for any damages arising from the use of this software.
//
// Permission is granted to anyone to use this software for any purpose, including commercial
// applications, and to alter it and redistribute it freely, subject to the following restrictions:
//
//     1. The origin of this software must not be misrepresented; you must not claim that you
//        wrote the original software. If you use this software in a product, an acknowledgment
//        in the product documentation would be appreciated but is not required.
//
//     2. Altered source versions must be plainly marked as such, and must not be misrepresented
//        as being the original software.
//
//     3. This notice may not be removed or altered from any source distribution.
//
// __________________________________________________________________________________________________
#ifndef __GOOEY_GLUT_INTERFACE_H__
#define __GOOEY_GLUT_INTERFACE_H__


#include "glgooey/services/interfaces/FrameworkInterface.h"
#include "glgooey/WindowManager.h"

#include "glgooey/core/KeyMap.h"


// *************************************************************************************************
namespace Gooey
{


// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
class GlutKeyMap
{
public:
    GlutKeyMap()
    {
        unsigned char i;
        for(i = 1; i <= 26;  ++i)
        {
                 if(i == 8) ucharMap_.addMapping( i, Key::backspace );
            else if(i == 9) ucharMap_.addMapping( i, Key::tab );
            else if(i == 13) ucharMap_.addMapping( i, Key::ret );
            else ucharMap_.addMapping(i, static_cast<Key::Code>(i+96));
        }
        ucharMap_.addMapping( 27,       Key::escape );
        for(i = 32; i <= 64;  ++i) ucharMap_.addMapping(i, static_cast<Key::Code>(i));
        for(i = 65; i <= 90;  ++i) ucharMap_.addMapping(i, static_cast<Key::Code>(i+32));
        for(i = 91; i <= 127; ++i) ucharMap_.addMapping(i, static_cast<Key::Code>(i));
        for(i = 0;  i <= 95;  ++i) ucharMap_.addMapping(i+160, static_cast<Key::Code>(i+160));
        
        intMap_.addMapping( GLUT_KEY_F1,        Key::F1 );
        intMap_.addMapping( GLUT_KEY_F2,        Key::F2 );
        intMap_.addMapping( GLUT_KEY_F3,        Key::F3 );
        intMap_.addMapping( GLUT_KEY_F4,        Key::F4 );
        intMap_.addMapping( GLUT_KEY_F5,        Key::F5 );
        intMap_.addMapping( GLUT_KEY_F6,        Key::F6 );
        intMap_.addMapping( GLUT_KEY_F7,        Key::F7 );
        intMap_.addMapping( GLUT_KEY_F8,        Key::F8 );
        intMap_.addMapping( GLUT_KEY_F9,        Key::F9 );
        intMap_.addMapping( GLUT_KEY_F10,       Key::F10 );
        intMap_.addMapping( GLUT_KEY_F11,       Key::F11 );
        intMap_.addMapping( GLUT_KEY_F12,       Key::F12 );
        intMap_.addMapping( GLUT_KEY_LEFT,      Key::left );
        intMap_.addMapping( GLUT_KEY_UP,        Key::up );
        intMap_.addMapping( GLUT_KEY_RIGHT,     Key::right );
        intMap_.addMapping( GLUT_KEY_DOWN,      Key::down );
        intMap_.addMapping( GLUT_KEY_PAGE_UP,   Key::pageUp );
        intMap_.addMapping( GLUT_KEY_PAGE_DOWN, Key::pageDown );
        intMap_.addMapping( GLUT_KEY_HOME,      Key::home );
        intMap_.addMapping( GLUT_KEY_END,       Key::end );
        intMap_.addMapping( GLUT_KEY_INSERT,    Key::insert );
    }                                             
                
    Key::Code toGooeyKey(unsigned char aKeyCode) const
    {
        return ucharMap_.toGooeyKey(aKeyCode);
    }
    
    Key::Code toGooeyKey(int aKeyCode) const
    {
        return intMap_.toGooeyKey(aKeyCode);
    }
    
    
private:
    KeyMap<unsigned char> ucharMap_;
    KeyMap<int> intMap_;
};                                                
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<





// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
class GlutInterface : public FrameworkInterface
{
public:
    //! Returns whether the shift key is currently pressed. 
    virtual bool isShiftPressed() const
    {
        return (glutGetModifiers() & GLUT_ACTIVE_SHIFT) != 0;
    }

    //! Returns whether the alt key is currently pressed. 
    virtual bool isAltPressed() const
    {
        return (glutGetModifiers() & GLUT_ACTIVE_ALT) != 0;
    }

    //! Returns whether the ctrl key is currently pressed.
    virtual bool isCtrlPressed() const
    {
        return (glutGetModifiers() & GLUT_ACTIVE_CTRL) != 0;
    }
    
    //! Handles a KeyDown Message
    bool onKeyDown(unsigned char aKey) const
    {
        return WindowManager::instance().onKeyDown(toGooeyKey(aKey));
    }    
    
    //! Handles a KeyUp Message
    bool onKeyUp(unsigned char aKey) const
    {
        return WindowManager::instance().onKeyUp(toGooeyKey(aKey));
    }

    //! Handles a SpecialKeyDown Message
    bool onSpecialKeyDown(int aKey) const
    {
        return WindowManager::instance().onKeyDown(toGooeyKey(aKey));
    }
    
    //! Handles a SpecialKeyUp Message
    bool onSpecialKeyUp(int aKey) const
    {
        return WindowManager::instance().onKeyUp(toGooeyKey(aKey));
    }
    
    //! Handles a MouseMove Message
    bool onMouseMove(int x, int y) const
    {
        return WindowManager::instance().onMouseMove(x, y);
    }

    //! Handles a MouseButton Message
    bool onMouseButton(int button, int state, int x, int y) const
    {
        switch(button)
        {
        case GLUT_LEFT_BUTTON:
            if(state == GLUT_UP)
                return WindowManager::instance().onLeftButtonUp(x, y);
            else
                return WindowManager::instance().onLeftButtonDown(x, y);
            break;
        case GLUT_MIDDLE_BUTTON:
            if(state == GLUT_UP)
                return WindowManager::instance().onMiddleButtonUp(x, y);
            else
                return WindowManager::instance().onMiddleButtonDown(x, y);
            break;
        case GLUT_RIGHT_BUTTON:
            if(state == GLUT_UP)
                return WindowManager::instance().onRightButtonUp(x, y);
            else
                return WindowManager::instance().onRightButtonDown(x, y);
            break;
        }

        return false;
    }

    
private:
    template <typename KeyType>    
    Key::Code toGooeyKey(const KeyType& aKeyCode) const
    {
        return keyMap_.toGooeyKey(aKeyCode);
    }

private:    
    GlutKeyMap keyMap_;
};
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<



// *************************************************************************************************
} // namespace Gooey



#endif
