#include "glgooey/core/Serialization.h"
#include "glgooey/services/interfaces/Archive.h"


// *************************************************************************************************
namespace Gooey
{


// *************************************************************************************************
namespace Core
{


ArchiveFactoryInterface* PoolInterface<ArchiveFactoryInterface>::firstElement_ = 0;


// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
Archive::Archive()
{
}
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<





// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
Archive::~Archive()
{
}
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<





// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
bool
Archive::open(const std::string& aFileName, bool isReading)
{
    isReadingFlag_ = isReading;

    // search for the last path seperator
    std::string::size_type pos1 = aFileName.find_last_of('/');
    std::string::size_type pos2 = aFileName.find_last_of('\\');
    
    // take the one that was found
    pos1 = (pos1 == std::string::npos)?pos2:pos1;
    
    if(pos1 == std::string::npos)
        baseDirectory_.clear();
    else
        baseDirectory_ = aFileName.substr(0, pos1+1);    

    return true;
}
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<





// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
std::string
Archive::convertFileName(const std::string& aFileName) const
{
    if(baseDirectory_.empty())
        return aFileName;
    else
        return baseDirectory_ + aFileName;
}
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<





// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
void
Archive::setClassName(const char* aClassName)
{
    if(aClassName!=0)
        className_ = aClassName;        
    else
        className_.clear();
}
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<





// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
ArchiveFactoryInterface::ArchiveFactoryInterface(const std::string& anArchiveDescription, const std::string anExtensionList, OperationMode anOperationMode) : 
    description_(anArchiveDescription), 
    extensionList_(anExtensionList), 
    operationMode_(anOperationMode)
{
}
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<





// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
Archive*
ArchiveFactoryInterface::createArchive(const std::string& aFileName, OperationMode anOperationMode)
{
    ArchiveFactoryInterface* a = findArchive(aFileName, anOperationMode);
    return a?a->create():0;
}
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<





// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
ArchiveFactoryInterface*
ArchiveFactoryInterface::findArchive(const std::string& aFileName, OperationMode anOperationMode)
{
    ArchiveFactoryInterface* archive = firstElement();
    while(archive != 0 && !archive->handles(aFileName, anOperationMode))
        archive = archive->nextElement();    

    return archive;
}
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<





// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
bool
ArchiveFactoryInterface::handlesExtension(const std::string& anExtension) const
{    
    std::string::const_iterator it = extensionList_.begin();
    std::string::const_iterator itTokenEnd = extensionList_.begin();
  
    while(it!=extensionList_.end())
    {        
        while(*it == '|') ++it;

        itTokenEnd = it;
        while(itTokenEnd!=extensionList_.end() && *itTokenEnd!='|') ++itTokenEnd;
        
        if(it<itTokenEnd)
            if(anExtension==std::string(it, itTokenEnd))
                return true;
        
        it = itTokenEnd;
    }

    return false;
}
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<





// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
bool
ArchiveFactoryInterface::handles(const std::string& aFileName, OperationMode anOperationMode)
{
    if((anOperationMode & operationMode_) != anOperationMode)
        return false;

    std::string::size_type pos = aFileName.find_last_of('.');
    if(pos == std::string::npos)
        return false;

    return handlesExtension(aFileName.substr(pos+1));
}
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<


// *************************************************************************************************
} // namespace Core

// *************************************************************************************************
} // namespace Gooey
